#!/usr/bin/python
#===============================================================================
# CERN (BE-CO-HT)
# Set converter board error register
#===============================================================================
# author: Theodor Stana (t.stana@cern.ch)
#         Maciej Suminski (maciej.suminski@cern.ch)
#
# date of creation: 2014-05-02
#
# version: 1.1
#
# description:
#   Set the converter board errors register, typically for clearing set error
#   bits. Note that typically to clear an error, one has to set it to logical
#   one.
#
# dependencies:
#   ../ei2c/ei2c.py
#   ../ei2c/ei2cdefine.py
#
#===============================================================================
# GNU LESSER GENERAL PUBLIC LICENSE
#===============================================================================
# This source file is free software; you can redistribute it and/or modify it
# under the terms of the GNU Lesser General Public License as published by the
# Free Software Foundation; either version 2.1 of the License, or (at your
# option) any later version. This source is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU Lesser General Public License for more details. You should have
# received a copy of the GNU Lesser General Public License along with this
# source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
#===============================================================================
# last changes:
#    2014-05-02    Theodor Stana     File created
#    2017-02-02    M. Suminski       Updated mem map to match v4 gateware,
#                                    renamed from setsr.py to seterr.py
#    2017-02-17    M. Suminski          Use SLOT configuration from ei2cdefine
#===============================================================================
#  TODO: -
#===============================================================================


import sys
sys.path.append("../ei2c")
from ei2c import *
import ei2cdefine
sys.path.append("..")
import convttl_regs as regs


if __name__ == "__main__":

    # Get the IP, user and password for the ELMA crate from either user input,
    # or ei2cdefine.py
    if (ei2cdefine.ENABLED):
        ip   = ei2cdefine.HNAME
        user = ei2cdefine.USER
        pwd  = ei2cdefine.PWD
        slot = ei2cdefine.SLOT
    else:
        ip   = raw_input("ELMA crate IP or hostname : ")
        user = raw_input("ELMA crate user name      : ")
        pwd  = raw_input("ELMA crate password       : ")
        slot = int(raw_input("ELMA crate slot number    : "))
    testelma = EI2C(ip, user, pwd)
    testelma.open()

    # Print which board is detected in the selected slot
    bid = testelma.get_bid(slot)

    # Get error register value, set and re-print
    v = testelma.read(slot, regs.ERR)
    print("ERR (pre-set)  : 0x%08x" % v)
    print("===========================")
    v = raw_input("Value to set   : 0x")
    v = int(v, 16)
    testelma.write(slot, regs.ERR, v)
    print("===========================")
    v = testelma.read(slot, regs.ERR)
    print("ERR (post-set) : 0x%08x" % v)

    testelma.close()
